// FlipFlopDlg.cpp : implementation file
//

#include "stdafx.h"
#include "TitleRunTimeTest.h"
#include "TitleRunTimeTestDlg.h"
#include "FlipFlopDlg.h"
#include "afxdialogex.h"
#include <atlpath.h>



// CFlipFlopDlg dialog

IMPLEMENT_DYNAMIC(CFlipFlopDlg, CDialog)

CFlipFlopDlg::CFlipFlopDlg(CWnd* pParent /*=NULL*/)
	: CDialog(IDD_FLIP_FLOP, pParent)
{

}

CFlipFlopDlg::~CFlipFlopDlg()
{
}

void CFlipFlopDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	DDX_Control(pDX, IDC_TIME_LINE, m_TimeLine);
}


BEGIN_MESSAGE_MAP(CFlipFlopDlg, CDialog)
	ON_BN_CLICKED(IDC_START, &CFlipFlopDlg::OnBnClickedStart)
	ON_BN_CLICKED(IDC_STOP, &CFlipFlopDlg::OnBnClickedStop)
	ON_BN_CLICKED(IDC_ABORT, &CFlipFlopDlg::OnBnClickedAbort)
	ON_BN_CLICKED(IDC_ADD, &CFlipFlopDlg::OnBnClickedAdd)
	ON_WM_TIMER()
	ON_WM_CLOSE()
	ON_LBN_SELCHANGE(IDC_TIME_LINE, &CFlipFlopDlg::OnLbnSelchangeTimeLine)
	ON_LBN_DBLCLK(IDC_TIME_LINE, &CFlipFlopDlg::OnLbnDblclkTimeLine)
	ON_BN_CLICKED(IDC_START_FIRST, &CFlipFlopDlg::OnBnClickedStartFirst)
END_MESSAGE_MAP()


// CFlipFlopDlg message handlers

static CString FormatTime(double time) {
	CString str;
	double intpart = 0;
	str.Format(_T("%d:%02d.%03d"), long(time / 60), long(time) % 60, long(modf(time, &intpart)*1000.0 /*+ 0.1*/));
	return str;

}

void CFlipFlopDlg::UpdateStatus()
{
	CString str;
	if (m_titleObject == nullptr || m_titlePreload == nullptr) {
		SetDlgItemText(IDC_STATUS, _T("Not Initialized"));
		SetDlgItemText(IDC_STATUS_INACTIVE, _T("Not Supported"));
		return;
	}
	SLTMRunTimeStatus status = SLTMRunTimeStatus_eNone;
	try {
		status = m_titleObject->Status;
		double currTime;
		double restTime;
		double allDuration;
		LONG loopped;
		m_titleObject->GetPlayInfo(&currTime, &restTime, &allDuration, &loopped);
		currTime += 0.0001;
		restTime += 0.0001;
		allDuration += 0.0001;
		SetDlgItemText(IDC_CURR_TIME, (LPCTSTR)FormatTime(currTime));
		SetDlgItemText(IDC_REST_TIME, (LPCTSTR)FormatTime(restTime));
		SetDlgItemText(IDC_DURATION, (LPCTSTR)FormatTime(allDuration));
		SetDlgItemText(IDC_STATUS, GetStatusStr(status));
		SetDlgItemText(IDC_TASK, (LPCTSTR)m_titleObject->TaskFileName);
		auto preloadStatus = m_titlePreload->PreloadStatus;
		SetDlgItemText(IDC_STATUS_INACTIVE, GetPreloadStatusStr(preloadStatus));
		SetDlgItemText(IDC_TASK_INACTIVE, (LPCTSTR)m_titlePreload->GetPreloadedTask());
		SetDlgItemText(IDC_DURATION_INACTIVE, (LPCTSTR)FormatTime(m_titlePreload->PreloadedDuration));

		if (m_bRunning) {
			m_bRunning = m_bRunning && (status >= SLTMRunTimeStatus_eStarting || status == SLTMRunTimeStatus_eStopping);
			if (m_bNextReady) {
				double fade = GetFade();
				double prestartDelta = fade + 1.000;
				if (restTime < prestartDelta && preloadStatus == SLTMPreloadStatus_ePrepared) {
					ATLTRACE(_T("Update Try Start restTime:%0.3f prestartDelta:%0.3f preloadStatus:%s\n"), restTime, prestartDelta, (LPCTSTR)GetPreloadStatusStr(preloadStatus));
					StartNext();
				}
			}
			else if (preloadStatus == SLTMPreloadStatus_eNone ) {
				CString file;
				auto index = GetNextFile(file);
				ATLTRACE(_T("UpdateStatus next PreloadTask(file:%s)\n"), (LPCTSTR)file);
				try {
					m_titlePreload->PreloadTask((LPCTSTR)file, GetMarkIn(), GetMarkOut());
					m_bNextReady = (index != LB_ERR);
					m_lastPreloadedIndex = index;
				}
				catch (_com_error& err) {
					if (m_bShowError) {
						ShowError(err);
						m_bShowError = false;
					}
					SetDlgItemText(IDC_PRELOAD_STATE, _T("error"));
					m_bNextReady = false;
					throw;
				}
			}
		}
		if (!m_bRunning) {
			preloadStatus = m_titlePreload->PreloadStatus;
			if (preloadStatus == SLTMPreloadStatus_eNone) {
				CString file;
				auto index = GetSelectedFile(file);
				ATLTRACE(_T("UpdateStatus current PreloadTask(file:%s)\n"), (LPCTSTR)file);
				try {
					m_titlePreload->PreloadTask((LPCTSTR)file, GetMarkIn(), GetMarkOut());
					m_bNextReady = (index != LB_ERR);
					m_lastPreloadedIndex = index;
				}
				catch (_com_error& err) {
					if (m_bShowError) {
						ShowError(err);
						m_bShowError = false;
					}
					SetDlgItemText(IDC_PRELOAD_STATE, _T("error"));
					m_bNextReady = false;
					throw;
				}
			}
		}
		SetDlgItemText(IDC_PLAY_STATE, m_bRunning? _T("Playing"): _T("Stopped"));
		SetDlgItemText(IDC_PRELOAD_STATE, m_bNextReady? _T("Next Ready") : _T("Next none"));

		m_bShowError = true;

	}
	catch (_com_error& err) {
		if (m_bShowError) {
			ShowError(err);
			m_bShowError = false;
		}
		SetDlgItemText(IDC_STATUS, _T("ERROR"));
	}
}

CString CFlipFlopDlg::GetStatusStr(SLTMRunTimeStatus status)
{
	switch (status) {
	case SLTMRunTimeStatus_eNone: return _T("None");
	case SLTMRunTimeStatus_ePreparing: return _T("Preparing");
	case SLTMRunTimeStatus_eStopped: return _T("Stopped");
	case SLTMRunTimeStatus_eStarting: return _T("Starting");
	case SLTMRunTimeStatus_eRunning: return _T("Running");
	case SLTMRunTimeStatus_eStopping: return _T("Stopping");
	//default: return _T("Unknown");
	}
	return _T("Unknown");
}

CString CFlipFlopDlg::GetPreloadStatusStr(SLTMPreloadStatus status)
{
	switch (status) {
	case SLTMPreloadStatus_eError: return _T("Error");
	case SLTMPreloadStatus_eNone: return _T("None");
	case SLTMPreloadStatus_eAssigned: return _T("Assigned");
	case SLTMPreloadStatus_ePrepared: return _T("Prepared");
	case SLTMPreloadStatus_eStarting: return _T("Starting");
	case SLTMPreloadStatus_eStopping: return _T("Stopping");
		//default: return _T("Unknown");
	}
	return _T("Unknown");
}

HRESULT CFlipFlopDlg::InitObject(ISLTMTitleRunTime3 * titleObject)
{
	HRESULT hr = S_OK;
	m_titleObject = titleObject;
	m_titlePreload = titleObject;
	if (m_titleObject == nullptr || m_titlePreload == nullptr)
		return E_INVALIDARG;

	//UpdateStatus();
	return hr;
}

void CFlipFlopDlg::OnBnClickedStart()
{
	StartNow();
}


void CFlipFlopDlg::OnBnClickedStop()
{
	if (m_titleObject == nullptr || m_titlePreload == nullptr)
		return;
	try {
		m_titleObject->Stop(0, GetFade());
		m_bRunning = false;
	}
	catch (_com_error& err) {
		ShowError(err);
	}
	UpdateStatus();
}


void CFlipFlopDlg::OnBnClickedAbort()
{
	if (m_titleObject == nullptr || m_titlePreload == nullptr)
		return;
	try {
		m_titleObject->Abort(GetFade());
		m_bRunning = false;
	}
	catch (_com_error& err) {
		ShowError(err);
	}
	UpdateStatus();
}


void CFlipFlopDlg::OnBnClickedAdd()
{
	if (m_titleObject == nullptr || m_titlePreload == nullptr)
		return;
	try {
		CString strFilters;
		CString strTaskName;
		try {
			strFilters = (LPCTSTR)m_titleObject->GetTaskFileMask();
			strTaskName = (LPCTSTR)m_titleObject->GetTaskFileName();
		}
		catch (_com_error& err) {
			ShowError(err);
			return;
		}

		strFilters += _T("All Files (*.*)|*.*||");

#define MAX_CFileDialog_FILE_COUNT 99
#define FILE_LIST_BUFFER_SIZE ((MAX_CFileDialog_FILE_COUNT * (MAX_PATH + 1)) + 1)

		CString fileName;
		LPTSTR p = fileName.GetBuffer(FILE_LIST_BUFFER_SIZE);

		CFileDialog fileDlg(TRUE, NULL, strTaskName, OFN_FILEMUSTEXIST | OFN_ENABLESIZING , strFilters, this);
		OPENFILENAME& ofn = fileDlg.GetOFN();
		ofn.Flags |= OFN_ALLOWMULTISELECT;
		ofn.lpstrFile = p;
		ofn.nMaxFile = FILE_LIST_BUFFER_SIZE;

		if (!strTaskName.IsEmpty()) {
			ATL::CPath path(strTaskName);
			path.RemoveFileSpec();
			m_strInitialTaskDir = (LPCTSTR)path;
		}
		ofn.lpstrInitialDir = m_strInitialTaskDir;
		// Display the file dialog. When user clicks OK, fileDlg.DoModal() 
		// returns IDOK.
		bool bOk = fileDlg.DoModal() == IDOK;
		fileName.ReleaseBuffer();

		if (bOk) {
			strTaskName = fileDlg.GetPathName();
			ATL::CPath path(strTaskName);
			path.RemoveFileSpec();
			m_strInitialTaskDir = (LPCTSTR)path;

			int curr = m_TimeLine.GetCurSel();
			if (curr == LB_ERR) {
				curr = m_TimeLine.GetCount();
			}
			else {
				curr++;
			}
			auto insert = curr;
			auto pos = fileDlg.GetStartPosition();
			while (pos != NULL) {
				auto fileName = fileDlg.GetNextPathName(pos);
				m_TimeLine.InsertString(insert++, fileName);
			}
			if (m_TimeLine.GetCount() > 0) {
				curr--;
				m_TimeLine.SetCurSel(curr);
				OnLbnSelchangeTimeLine();
			}
		}
	}
	catch (_com_error& err) {
		ShowError(err);
	}
	UpdateStatus();
}


void CFlipFlopDlg::OnTimer(UINT_PTR nIDEvent)
{
	int static reenter = 0;
	if (reenter > 0) return;

	reenter++;
	try {
		UpdateStatus();
		CDialog::OnTimer(nIDEvent);
	}
	catch (_com_error& err) {
		ShowError(err);
	}
	catch (...) {
		ShowError(_com_error(E_FAIL));
	}
	reenter--;
}


BOOL CFlipFlopDlg::OnInitDialog()
{
	CDialog::OnInitDialog();


	SetDlgItemInt(IDC_CRAOSS_FADE, 500);
	SetDlgItemInt(IDC_MARK_IN, 0);
	SetDlgItemInt(IDC_MARK_OUT, 1000);

	m_TimerId = SetTimer(1, 200, NULL);

	return TRUE;  // return TRUE unless you set the focus to a control
				  // EXCEPTION: OCX Property Pages should return FALSE
}


void CFlipFlopDlg::OnClose()
{
	KillTimer(m_TimerId);
	m_TimerId = 0;

	CDialog::OnClose();
}


void CFlipFlopDlg::OnLbnSelchangeTimeLine()
{
	if (m_titleObject == nullptr || m_titlePreload == nullptr)
		return;
	try {
		CString file;
		m_bNextReady = false;
		auto curr = GetSelectedFile(file);
		ATLTRACE(_T("CFlipFlopDlg::OnLbnSelchangeTimeLine PreloadTask(file:%s)\n"), (LPCTSTR)file);
		m_titlePreload->PreloadTask((LPCTSTR)file, GetMarkIn(), GetMarkOut());
		m_bNextReady = (curr != LB_ERR);
		m_lastPreloadedIndex = curr;
	}
	catch (_com_error& err) {
		ShowError(err);
	}
	UpdateStatus();

}

int CFlipFlopDlg::GetSelectedFile(CString &file)
{
	auto curr = m_TimeLine.GetCurSel();
	if (curr == LB_ERR) {
		if (m_TimeLine.GetCount() <= 0) {
			return LB_ERR;
		}
		curr = 0;
		m_TimeLine.SetCurSel(curr);
	}
	LPTSTR buffer = file.GetBufferSetLength(m_TimeLine.GetTextLen(curr));
	m_TimeLine.GetText(curr, buffer);
	file.ReleaseBuffer();
	return curr;
}

int CFlipFlopDlg::GetNextFile(CString & file)
{
	auto curr = m_TimeLine.GetCurSel();
	if (curr == LB_ERR) {
		return curr;
	}
	if (++curr >= m_TimeLine.GetCount())
		return LB_ERR;
	LPTSTR buffer = file.GetBufferSetLength(m_TimeLine.GetTextLen(curr));
	m_TimeLine.GetText(curr, buffer);
	file.ReleaseBuffer();
	return curr;
}

void CFlipFlopDlg::StartNext()
{
	if (m_titleObject == nullptr || m_titlePreload == nullptr)
		return;
	try {
		auto fade = GetFade();
		auto status = m_titleObject->Status;
		double currTime;
		double restTime;
		double allDuration;
		LONG loopped;
		double startTime = 0.0;
		m_titleObject->GetPlayInfo(&currTime, &restTime, &allDuration, &loopped);
		if (status >= SLTMRunTimeStatus_eStarting || status == SLTMRunTimeStatus_eStopping) {
			fade = min(fade, restTime);
			startTime = currTime + restTime - fade;
		}

		ATLTRACE(_T("CFlipFlopDlg::StartNext(time:%f)\n"), startTime);
		m_titlePreload->StartPreloaded(startTime, fade);
		if (status >= SLTMRunTimeStatus_eStarting || status == SLTMRunTimeStatus_eStopping) {
			m_TimeLine.SetCurSel(m_lastPreloadedIndex);
		}
		m_bRunning = true;
		m_bNextReady = false;

	}
	catch (_com_error& err) {
		ShowError(err);
		if (err.Error() == E_POINTER) {
			SetDlgItemText(IDC_STATUS, _T("ERROR"));
		}
	}
}

void CFlipFlopDlg::StartNow()
{
	if (m_titleObject == nullptr || m_titlePreload == nullptr)
		return;
	try {
		ATLTRACE(_T("CFlipFlopDlg::StartNow()\n"));
		auto status = m_titleObject->Status;
		auto fade = GetFade();
		m_titlePreload->StartPreloaded(0, fade);
		if (status >= SLTMRunTimeStatus_eStarting || status == SLTMRunTimeStatus_eStopping) {
			m_TimeLine.SetCurSel(m_lastPreloadedIndex);
		}
		m_bRunning = true;
		m_bNextReady = false;
	}
	catch (_com_error& err) {
		ShowError(err);
		if (err.Error() == E_POINTER) {
			SetDlgItemText(IDC_STATUS, _T("ERROR"));
		}
	}
}


void CFlipFlopDlg::OnLbnDblclkTimeLine()
{
	// TODO: Add your control notification handler code here
}


void CFlipFlopDlg::OnBnClickedStartFirst()
{
	if (m_titleObject == nullptr || m_titlePreload == nullptr)
		return;
	try {
		m_titleObject->Start(0, GetFade());
		m_bRunning = true;
	}
	catch (_com_error& err) {
		ShowError(err);
	}
	UpdateStatus();
}
