﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows.Controls;
using Microsoft.Win32;
using System.Globalization;
using System.Windows;

namespace SLStreamerPro
{
	class ControlSerializer
	{
		public static void SaveForm(Window window, RegistryKey key, bool bSavePos)
		{
			try
			{
				if (window.WindowState == WindowState.Minimized)
					return;
				if (bSavePos)
				{
					key.SetValue("Left", window.Left.ToString(CultureInfo.InvariantCulture));
					key.SetValue("Top", window.Top.ToString(CultureInfo.InvariantCulture));
				}

				key.SetValue("Width", window.Width.ToString(CultureInfo.InvariantCulture));
				key.SetValue("Height", window.Height.ToString(CultureInfo.InvariantCulture));
				key.SetValue("WindowState", window.WindowState.ToString());
			}
			catch
			{
			}
		}

		public static void SaveGrid(Grid grid, RegistryKey key, string subKey)
		{
            try
            {
                using (RegistryKey ctrlKey = key.CreateSubKey(subKey))
                {
                    ctrlKey.SetValue("Columns", grid.ColumnDefinitions.Count.ToString());
                    for (int i = 0; i < grid.ColumnDefinitions.Count; i++)
                    {
                        try
                        {
                            ColumnDefinition def = grid.ColumnDefinitions[i];
                            ctrlKey.SetValue("ColumnType" + i.ToString(), def.Width.GridUnitType.ToString());
                            ctrlKey.SetValue("ColumnWidth" + i.ToString(), def.Width.Value.ToString(CultureInfo.InvariantCulture));
                        }
                        catch
                        {
                        }
                    }
                    ctrlKey.SetValue("Rows", grid.RowDefinitions.Count.ToString());
                    for (int i = 0; i < grid.RowDefinitions.Count; i++)
                    {
                        try
                        {
                            RowDefinition def = grid.RowDefinitions[i];
                            ctrlKey.SetValue("RowType" + i.ToString(), def.Height.GridUnitType.ToString());
                            ctrlKey.SetValue("RowWidth" + i.ToString(), def.Height.Value.ToString(CultureInfo.InvariantCulture));
                        }
                        catch
                        {
                        }
                    }
                }
            }
            catch
            {
            }
            
		}

		public static void SaveListView(ListView listView, RegistryKey key, string subKey)
		{
			using (RegistryKey ctrlKey = key.CreateSubKey(subKey))
			{
				GridView view = listView.View as GridView;
				if (view == null)
					return;
                for (int i = 0; i < view.Columns.Count; i++)
                {
                    try
                    {
                        ctrlKey.SetValue("Column" + i.ToString(), view.Columns[i].Width.ToString(CultureInfo.InvariantCulture));
                    }
                    catch
                    {
                    }
                }
			}
		}

		public static void LoadForm(Window window, RegistryKey key, bool bLoadPos)
		{
			try
			{
				if (bLoadPos)
				{
					window.Left = Math.Max(0, Convert.ToDouble(key.GetValue("Left",
						window.Left.ToString(CultureInfo.InvariantCulture)), CultureInfo.InvariantCulture));
					window.Top = Math.Max(0, Convert.ToDouble(key.GetValue("Top",
						window.Top.ToString(CultureInfo.InvariantCulture)), CultureInfo.InvariantCulture));
				}

				window.Width = Math.Min(System.Windows.SystemParameters.PrimaryScreenWidth,
					Convert.ToDouble(key.GetValue("Width", window.Width.ToString(CultureInfo.InvariantCulture)),
					CultureInfo.InvariantCulture));
				window.Height = Math.Min(System.Windows.SystemParameters.PrimaryScreenWidth,
					Convert.ToDouble(key.GetValue("Height", window.Height.ToString(CultureInfo.InvariantCulture)),
					CultureInfo.InvariantCulture));
				window.WindowState = (WindowState)Enum.Parse(typeof(WindowState), key.GetValue("WindowState", window.WindowState.ToString()) as string);
			}
			catch
			{
			}
		}

		public static void LoadGrid(Grid grid, RegistryKey key, string subKey)
		{
			using (RegistryKey ctrlKey = key.OpenSubKey(subKey))
			{
				for (int i = 0; i < grid.ColumnDefinitions.Count; i++)
				{
					try
					{

						GridUnitType type = (GridUnitType)Enum.Parse(typeof(GridUnitType),
							ctrlKey.GetValue("ColumnType" + i.ToString()) as string);
						double width = Convert.ToDouble(ctrlKey.GetValue("ColumnWidth" + i.ToString()) as string,
							CultureInfo.InvariantCulture);
						grid.ColumnDefinitions[i].Width = new GridLength(width, type);
					}
					catch
					{

					}
				}
				for (int i = 0; i < grid.RowDefinitions.Count; i++)
				{
					try
					{

						GridUnitType type = (GridUnitType)Enum.Parse(typeof(GridUnitType),
							ctrlKey.GetValue("RowType" + i.ToString()) as string);
						double width = Convert.ToDouble(ctrlKey.GetValue("RowWidth" + i.ToString()) as string,
							CultureInfo.InvariantCulture);
						grid.RowDefinitions[i].Height = new GridLength(width, type);
					}
					catch
					{

					}
				}
			}
		}

		public static void LoadListView(ListView listView, RegistryKey key, string subKey)
		{
			using (RegistryKey ctrlKey = key.OpenSubKey(subKey))
			{
				GridView view = listView.View as GridView;
				if (view == null)
					return;
                for (int i = 0; i < view.Columns.Count; i++)
                {
                    try
                    {
                        view.Columns[i].Width = Convert.ToDouble(ctrlKey.GetValue("Column" + i.ToString(), view.Columns[i].Width.ToString(CultureInfo.InvariantCulture)), CultureInfo.InvariantCulture);
                    }
                    catch
                    {
                    }
                }
			}
		}
	}
}
