﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Collections;
using System.ComponentModel;
using System.Xml;

namespace SLStreamerPro.PropertyGrid
{
    class EnumValue
    {
        string name;
        string value;
		
		public EnumValue(XmlNode xmlEnum)
        {
			name = XmlHelper.GetAttributeByName(xmlEnum, XmlTags.Name).Value;
			value = XmlHelper.GetAttributeByName(xmlEnum, XmlTags.Value).Value;
        }

        public string Name
        {
            get { return name; }
        }

        public string Value
        {
            get { return value; }
        }

        internal void Save(XmlDocument xmlDoc, XmlNode xmlEnum)
        {
            XmlHelper.AppendAttribute(xmlDoc, xmlEnum, XmlTags.Name, name);
			XmlHelper.AppendAttribute(xmlDoc, xmlEnum, XmlTags.Value, value);
        }
    }

    class EnumType
    {
        string name;
        List<EnumValue> values;

        private EnumType()
        {
            name = "";
            values = new List<EnumValue>();
        }

        public static EnumType Create(XmlNode xmlType)
        {
            EnumType type = new EnumType();
            foreach (XmlNode xmlEnum in xmlType.ChildNodes)
            {
                if (xmlEnum.NodeType != XmlNodeType.Element || string.Compare(xmlEnum.Name, XmlTags.Enum, true) != 0)
                    continue;
				type.values.Add(new EnumValue(xmlEnum));
            }
            if (type.values.Count < 1)
                return null;
            return type;
        }

        public string Name
        {
            get { return name; }
        }

        public void Save(XmlDocument xmlDoc, XmlElement xmlType)
        {
            foreach (EnumValue value in values)
            {
                XmlNode xmlEnum = xmlType.AppendChild(xmlDoc.CreateElement(XmlTags.Enum));
                value.Save(xmlDoc, xmlEnum);
            }
        }

        public string GetValueByName(string name)
        {
            foreach (EnumValue value in values)
                if (string.Compare(value.Name, name, true) == 0)
                    return value.Value;
            return "";
        }

        public string GetTextByValue(string value)
        {
            foreach (EnumValue v in values)
                if (string.Compare(v.Value, value, true) == 0)
                    return v.Name;
            return "";
        }

        public List<EnumValue> Values
        {
            get { return values; }
        }

        internal string[] GetTextArray()
        {
            List<string> names = new List<string>();
            foreach (EnumValue value in values)
                names.Add(value.Name);
            return names.ToArray();
        }
    }
}