﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Collections;
using System.ComponentModel;
using System.Xml;
using SLGraphClient;

namespace SLStreamerPro.PropertyGrid
{
	class ObjectClass : CollectionBase, ICustomTypeDescriptor
	{
		XmlDocument xmlDoc;
		ObjectProperty defaultProperty;
		string _sectionTag;

		public ObjectClass(string xmlString, string sectionTag, bool isReadOnly)
		{
			_sectionTag = sectionTag;
			xmlDoc = new XmlDocument();
			defaultProperty = null;
			int categoryIndex = 0;
			try
			{
				xmlDoc.LoadXml("<" + XmlTags.Root + ">" + xmlString + "</" + XmlTags.Root + ">");
				XmlNode xmlRoot = XmlHelper.GetFirtsElementByName(xmlDoc.ChildNodes, XmlTags.Root);
				foreach (XmlNode xmlSettings in xmlRoot.ChildNodes)
				{
					if (xmlSettings.NodeType != XmlNodeType.Element || string.Compare(xmlSettings.Name, _sectionTag, true) != 0)
						continue;
					XmlAttribute idAttr = XmlHelper.GetAttributeByName(xmlSettings, XmlTags.Id);
					if (idAttr == null)
						continue;
					XmlAttribute nameAttr = XmlHelper.GetAttributeByName(xmlSettings, XmlTags.Name);
					ObjectCategory category = null;
					if (nameAttr != null)
						category = new ObjectCategory(idAttr.Value, nameAttr.Value);
					else
						category = new ObjectCategory(idAttr.Value, "");
					categoryIndex++;

					Add(category);
					foreach (XmlNode xmlParam in xmlSettings.ChildNodes)
					{
						if (xmlParam.NodeType != XmlNodeType.Element || string.Compare(xmlParam.Name, XmlTags.Param, true) != 0)
							continue;
						idAttr = XmlHelper.GetAttributeByName(xmlParam, XmlTags.ParamID);
						if (idAttr == null)
							continue;
						nameAttr = XmlHelper.GetAttributeByName(xmlParam, XmlTags.Name);
						XmlAttribute valueAttr = XmlHelper.GetAttributeByName(xmlParam, XmlTags.Value);
						XmlAttribute descriptionAttr = XmlHelper.GetAttributeByName(xmlParam, XmlTags.Info);
						if (nameAttr == null || valueAttr == null)
							continue;

						string value = valueAttr.Value;
						EnumType enumType = EnumType.Create(xmlParam);
						if (enumType != null)
							value = enumType.GetTextByValue(value);
						ObjectProperty property = null;
						if (descriptionAttr == null)
							property = new ObjectProperty(idAttr.Value, nameAttr.Value, value, enumType, "", categoryIndex.ToString() + " " + category.Text, isReadOnly, true);
						else
							property = new ObjectProperty(idAttr.Value, nameAttr.Value, value, enumType, descriptionAttr.Value, categoryIndex.ToString() + " " + category.Text, isReadOnly, true);
						category.Add(property);
						if (defaultProperty == null)
							defaultProperty = property;
					}
				}
			}
			catch
			{
			}
		}

		public void Add(ObjectCategory Value)
		{
			base.List.Add(Value);
		}

		public ObjectCategory this[int index]
		{
			get { return base.List[index] as ObjectCategory; }
			set { base.List[index] = value as ObjectCategory; }
		}

		public string GetXml()
		{
			try
			{
				XmlNode xmlRoot = XmlHelper.GetFirtsElementByName(xmlDoc.ChildNodes, XmlTags.Root);
				if (xmlRoot == null)
					return null;
				string cfgString = "";

				for (int i = 0; i < this.Count; i++)
				{
					ObjectCategory category = this[i] as ObjectCategory;
					foreach (XmlNode xmlSettings in xmlRoot.ChildNodes)
					{
						if (xmlSettings.NodeType != XmlNodeType.Element || string.Compare(xmlSettings.Name, _sectionTag, true) != 0)
							continue;
						XmlAttribute idAttr = XmlHelper.GetAttributeByName(xmlSettings, XmlTags.Id);
						if ((idAttr != null && category.Id != idAttr.Value) || (idAttr == null && category.Id != null))
							continue;

						for (int j = 0; j < category.Count; j++)
						{
							ObjectProperty property = category[j] as ObjectProperty;
							foreach (XmlNode xmlParam in xmlSettings.ChildNodes)
							{
								if (xmlParam.NodeType != XmlNodeType.Element || string.Compare(xmlParam.Name, XmlTags.Param, true) != 0)
									continue;
								idAttr = XmlHelper.GetAttributeByName(xmlParam, XmlTags.ParamID);
								if (idAttr == null || string.Compare(idAttr.Value, property.Id, true) != 0)
									continue;
								if (property.Type == null)
									XmlHelper.AppendAttribute(xmlDoc, xmlParam, XmlTags.Value, property.Value as string);
								else
									XmlHelper.AppendAttribute(xmlDoc, xmlParam, XmlTags.Value,
										property.Type.GetValueByName(property.Value as string));
								break;
							}
						}
						XmlDocument xmlDocSettings = new XmlDocument();
						XmlNode xmlSts = xmlDocSettings.AppendChild(xmlDocSettings.CreateElement(_sectionTag));

						XmlHelper.CopyNode(xmlDocSettings, xmlSts, xmlSettings);
						cfgString += xmlDocSettings.InnerXml;
						break;
					}
				}
				return cfgString;
			}
			catch
			{
			}
			return null;
		}

		#region ICustomTypeDescriptor Members

		public AttributeCollection GetAttributes()
		{
			return TypeDescriptor.GetAttributes(this, true);
		}

		public string GetClassName()
		{
			return TypeDescriptor.GetClassName(this, true);
		}

		public string GetComponentName()
		{
			return TypeDescriptor.GetComponentName(this, true);
		}

		public TypeConverter GetConverter()
		{
			return TypeDescriptor.GetConverter(this, true);
		}

		public EventDescriptor GetDefaultEvent()
		{
			return TypeDescriptor.GetDefaultEvent(this, true);
		}

		public PropertyDescriptor GetDefaultProperty()
		{
			if (defaultProperty == null)
				return null;
			return new ObjectPropertyDescriptor(defaultProperty, null);
		}

		public object GetEditor(Type editorBaseType)
		{
			return TypeDescriptor.GetEditor(this, editorBaseType, true);
		}

		public EventDescriptorCollection GetEvents(Attribute[] attributes)
		{
			return TypeDescriptor.GetEvents(this, attributes, true);
		}

		public EventDescriptorCollection GetEvents()
		{
			return TypeDescriptor.GetEvents(this, true);
		}

		public PropertyDescriptorCollection GetProperties(Attribute[] attributes)
		{
			List<PropertyDescriptor> newProps = new List<PropertyDescriptor>();
			for (int i = 0; i < this.Count; i++)
			{
				ObjectCategory category = this[i] as ObjectCategory;
				for (int j = 0; j < category.Count; j++)
				{
					ObjectProperty property = category[j] as ObjectProperty;
					newProps.Add(new ObjectPropertyDescriptor(property, attributes));
				}
			}
			return new PropertyDescriptorCollection(newProps.ToArray());
		}

		public PropertyDescriptorCollection GetProperties()
		{
			return GetProperties(null);
		}

		public object GetPropertyOwner(PropertyDescriptor pd)
		{
			return this;
		}

		#endregion
	}
}
