﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.ComponentModel;

namespace SLStreamerPro.PropertyGrid
{
    class PropertyTypeConverter : StringConverter
    {
        string[] values;

        public PropertyTypeConverter(string[] values)
        {
            if (values == null)
                this.values = null;
            else
            {
                this.values = new string[values.Length];
                values.CopyTo(this.values, 0);
            }
        }

        public override bool GetStandardValuesSupported(ITypeDescriptorContext context)
        {
            return (values != null);
        }

        public override bool GetStandardValuesExclusive(ITypeDescriptorContext context)
        {
            return (values != null);
        }

        public override TypeConverter.StandardValuesCollection GetStandardValues(ITypeDescriptorContext context)
        {
            if (values == null)
                return null;
            return new StandardValuesCollection(values);
        }
    }

    class ObjectPropertyDescriptor : PropertyDescriptor
    {
        ObjectProperty property;
        PropertyTypeConverter typeConverter;

        public ObjectPropertyDescriptor(ObjectProperty property, Attribute[] attrs)
            : base(property.Name, attrs)
        {
            this.property = property;
            if (property.Type == null)
                typeConverter = new PropertyTypeConverter(null);
            else
                typeConverter = new PropertyTypeConverter(property.Type.GetTextArray());
        }

        public override TypeConverter Converter
        {
            get { return typeConverter; }
        }

        public override bool CanResetValue(object component)
        {
            return false;
        }

        public override Type ComponentType
        {
            get { return null; }
        }

        public override object GetValue(object component)
        {
            return property.Value;
        }

        public override string Description
        {
            get { return property.Description; }
        }

        public override bool IsReadOnly
        {
            get { return property.IsReadOnly; }
        }

        public override Type PropertyType
        {
            get { return property.Value.GetType(); }
        }

        public override void ResetValue(object component)
        {
        }

        public override void SetValue(object component, object value)
        {
            property.Value = value;
        }

        public override bool ShouldSerializeValue(object component)
        {
            return false;
        }

        public override string Category
        {
            get { return property.Category; }
        }
    }
}
