﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;
using System.Windows.Forms;
using SLStreamerPro.PropertyGrid;
using SLGraphClient;
using System.Windows.Threading;
using Microsoft.Win32;

namespace SLStreamerPro
{
	/// <summary>
	/// Interaction logic for StatisticsWindow.xaml
	/// </summary>
	public partial class StatisticsWindow : Window
	{
		System.Windows.Forms.PropertyGrid _propertyGrid;
		ISLGComponent _component;
		DispatcherTimer _timer = new DispatcherTimer();

		public StatisticsWindow(ISLGComponent component, string statistics)
		{
			InitializeComponent();

			_component = component;
			_propertyGrid = new System.Windows.Forms.PropertyGrid();
			_propertyGrid.PropertySortChanged += new EventHandler(_propertyGrid_PropertySortChanged);
			_propertyGridHost.Child = _propertyGrid;

            ObjectClass stat = new ObjectClass(statistics, XmlTags.Statistics, false);
			_propertyGrid.SelectedObject = stat;
			_propertyGrid.PropertySort = PropertySort.Categorized;
			_propertyGrid.Refresh();

			// Modify toolstrip
			foreach (System.Windows.Forms.Control control in _propertyGrid.Controls)
			{
				ToolStrip toolStrip = control as ToolStrip;
				if (toolStrip == null)
					continue;

				for (int i = 0; i < 2 && i < toolStrip.Items.Count; i++)
					toolStrip.Items[i].Enabled = false;
				break;
			}
			try
			{
				using (RegistryKey key = Registry.CurrentUser.OpenSubKey("Software\\SoftLab-NSK\\SLStreamerTest\\StatisticsWindow"))
				{
					ControlSerializer.LoadForm(this, key, false);
				}
			}
			catch
			{
			}
			_timer.Interval = TimeSpan.FromSeconds(1);
			_timer.Tick += new EventHandler(_timer_Tick);
			_timer.Start();
		}

		private void Window_Closing(object sender, System.ComponentModel.CancelEventArgs e)
		{
			_timer.Stop();
			try
			{
				using (RegistryKey key = Registry.CurrentUser.CreateSubKey("Software\\SoftLab-NSK\\SLStreamerTest\\StatisticsWindow"))
				{
					ControlSerializer.SaveForm(this, key, false);
				}
			}
			catch
			{
			}
		}

		private void _propertyGrid_PropertySortChanged(object sender, EventArgs e)
		{
			if (_propertyGrid.PropertySort == PropertySort.CategorizedAlphabetical)
				_propertyGrid.PropertySort = PropertySort.Categorized;
			else if (_propertyGrid.PropertySort == PropertySort.Alphabetical)
				_propertyGrid.PropertySort = PropertySort.NoSort;
		}

		private void _timer_Tick(object sender, EventArgs e)
		{
			if (_updateCheckBox.IsChecked != true)
				return;
			string statistics;
			if (_component.GetStatistics(out statistics) == HR.S_OK)
			{
                ObjectClass stat = new ObjectClass(statistics, XmlTags.Statistics, false);
				_propertyGrid.SelectedObject = stat;
				_propertyGrid.PropertySort = PropertySort.Categorized;
				_propertyGrid.Refresh();
			}
		}

		private void Window_Loaded(object sender, RoutedEventArgs e)
		{
			if (this.Left < 0)
				this.Left = 0;
			if (this.Top < 0)
				this.Top = 0;
		}
	}
}
