﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Xml;
using System.IO;

namespace SLStreamerPro
{
    public class XmlHelper
    {
        public static string GetChildsXml(XmlNode node, string childName)
        {
            string xml = null;
            foreach (XmlNode xmlNode in node.ChildNodes)
            {
                if (xmlNode.NodeType != XmlNodeType.Element || string.Compare(xmlNode.Name, childName) != 0)
                    continue;
                XmlDocument xmlDoc = new XmlDocument();
                XmlNode xmlNode2 = xmlDoc.AppendChild(xmlDoc.CreateElement(xmlNode.Name));
                XmlHelper.CopyNode(xmlDoc, xmlNode2, xmlNode);
                if (xml == null)
                    xml = xmlDoc.InnerXml;
                else
                    xml += "\n" + xmlDoc.InnerXml;
            }
            return xml;
        }

        public static string WriteToString(XmlDocument xmlDoc)
        {
            StringWriter writer = new StringWriter();
            xmlDoc.Save(writer);
            return writer.ToString();
        }

        public static void AppendAttribute(XmlDocument xmlDoc, XmlNode node, string name, string value)
        {
            XmlAttribute attr = GetAttributeByName(node, name);
            if (attr == null)
                attr = node.Attributes.Append(xmlDoc.CreateAttribute(name));
            attr.Value = value;
        }

        public static XmlAttribute GetAttributeByName(XmlNode node, string name)
        {
            foreach (XmlAttribute xmlAttr in node.Attributes)
            {
                if (string.Compare(xmlAttr.Name, name, true) != 0)
                    continue;
                return xmlAttr;
            }
            return null;
        }

        public static XmlNode GetFirtsElementByName(XmlNodeList nodeList, string name)
        {
            foreach (XmlNode xmlNode in nodeList)
            {
                if (xmlNode.NodeType != XmlNodeType.Element || string.Compare(xmlNode.Name, name, true) != 0)
                    continue;
                return xmlNode;
            }
            return null;
        }

        public static void AppendNode(XmlDocument dstDoc, XmlNode dstParentNode, XmlNode srcNode)
        {
            if (srcNode.NodeType != XmlNodeType.Element)
                return;
            XmlNode dstNode = dstParentNode.AppendChild(dstDoc.CreateElement(srcNode.Name));
            foreach (XmlAttribute attr in srcNode.Attributes)
                XmlHelper.AppendAttribute(dstDoc, dstNode, attr.Name, attr.Value);

            foreach (XmlNode subNode in srcNode.ChildNodes)
            {
                if (subNode.NodeType != XmlNodeType.Element)
                    continue;
                AppendNode(dstDoc, dstNode, subNode);
            }
        }

        public static void CopyNode(XmlDocument dstDoc, XmlNode dstNode, XmlNode srcNode)
        {
            if (srcNode.NodeType != XmlNodeType.Element || dstNode.NodeType != XmlNodeType.Element)
                return;
            foreach (XmlAttribute attr in srcNode.Attributes)
                XmlHelper.AppendAttribute(dstDoc, dstNode, attr.Name, attr.Value);
            
            foreach (XmlNode subNode in srcNode.ChildNodes)
            {
                if (subNode.NodeType != XmlNodeType.Element)
                    continue;
                AppendNode(dstDoc, dstNode, subNode);
            }
        }

        public static bool GetUintAttribute(XmlNode node, string name, out uint value)
        {
            value = 0;
            try
            {
                XmlAttribute attr = GetAttributeByName(node, name);
                value = Convert.ToUInt32(attr.Value);
                return true;
            }
            catch
            {
            }
            return false;
        }
    }
}
