﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using SLTMTitleRunTime;


namespace FDTitleCSharp {
    /// <summary>
    /// Демонстрация работы с титровальными элементами
    /// 
    ///</summary>
    public partial class ElementsForm: Form {
        public ElementsForm() {
            InitializeComponent();
        }
        ISLTMTitleRunTime3 _iTitleObject = null;
        ISLTMTitleElement _iCurrTitleElement = null;
        ISLTMTitleEnumElements _iTitleEnum = null;

		private void ClearAll()
		{
			lbElements.Items.Clear();
			_iTitleObject = null;
			_iCurrTitleElement = null;
			_iTitleEnum = null;
		}

	
		public ElementsForm(ISLTMTitleRunTime3 iTitleObject)
		{
            InitializeComponent();
            _iTitleObject = iTitleObject;
            SecondInitialize();
        }

        /// <summary>
        /// Инициализация списка титровальных элементов
        /// </summary>
        private void SecondInitialize() {
            if (_iTitleObject == null)
                return;
            tbName.Text = _iTitleObject.Name;
            try {
                // обект реализующий интерфейс ISLTMTitleRunTime3 (TitleObject)
                // реализует также интерфейс перечисления титровальных элементов (ISLTMTitleEnumElements)
                _iTitleEnum = _iTitleObject as ISLTMTitleEnumElements;
                if (_iTitleEnum == null)
                    return;
                tbNumElements.Text = _iTitleEnum.NumObjects.ToString();
                lbElements.Items.Clear();
                // перечислим имена всех титровальных элементов
                for (int i = 0; i < _iTitleEnum.NumObjects; i++) {
                    lbElements.Items.Add(_iTitleEnum.GetName(i));
                }
                if (_iTitleEnum.NumObjects > 0) {
                    lbElements.SelectedIndex = 0; // генерирует событие lbElements_SelectedIndexChanged 
                }
            }
            catch (Exception err) {
                MessageBox.Show(this, err.Message);
                lbElements.Items.Clear();
            }

        }

        private void lbElements_SelectedIndexChanged(object sender, EventArgs e) {
            if (_iTitleEnum != null) {
                try {
                    // запомним выбранный титровальный элемент
                    _iCurrTitleElement = (ISLTMTitleElement)_iTitleEnum.GetElement(lbElements.SelectedIndex);
                    tbCurrName.Text = _iCurrTitleElement.Name;
                    tbTask.Text = _iCurrTitleElement.GetTask();
                    UpdateProperties();
                }
                catch (Exception err) {
                    MessageBox.Show(err.Message);
                }
            }

        }

        volatile int reenter = 0;
        private void timer1_Tick(object sender, EventArgs e) {
            if (_iCurrTitleElement == null)
                return;
            try {
                if (++reenter > 1)
                    return;
                tbGlobalTime.Text = _iCurrTitleElement.GlobalTime.ToString("F2");
                tbStatus.Text = _iCurrTitleElement.Status.ToString();
                // for SLTMTitleRunTime.dll ver 4.5.x.x
                // усли версия вашего рантайма меньше чем 4.5 
                // закоменнтируйте следующие строки ...
                var element = _iCurrTitleElement as ISLTMTitleElement2; 
                if (element != null) {
                    var info = new PlayInfo();
                    element.GetPlayInfo(ref info);
                    tbRestTime.Text = info.restTime.ToString("F2");
                    tbDuration.Text = info.duration.ToString("F2");
                    if (info.loopped) {
                        tbLoopped.Text = "Loopped";
                    }
                    else {
                        tbLoopped.Text = "Not Loopped";
                    }
                }
                // ...до этого места
            }
            catch { }
            finally {
                reenter--;
            }
        }

        private void UpdateProperties() {
            if (_iCurrTitleElement == null)
                return;
            try {
                int[] arr = null;//new int[2];
                // возьмём стандартное свойство с именем "Location"
                arr = _iCurrTitleElement["Location"] as int[];
                if (arr != null && arr.Length == 2) {
                    udLocationX.Value = arr[0];
                    udLocationY.Value = arr[1];
                }
                // возьмём стандартное свойство с именем "Size"
                arr = _iCurrTitleElement["Size"] as int[];
                if (arr != null && arr.Length == 2) {
                    udSizeW.Value = arr[0];
                    udSizeH.Value = arr[1];
                }
                ShowProperties_Click(null, null);
            }
            catch { }

        }

        private void BrowseTask_Click(object sender, EventArgs e) {
            if (_iCurrTitleElement == null)
                return;
            openFileDialog1.Filter = _iCurrTitleElement.GetTaskMask() + "All Files|*.*";
            openFileDialog1.FileName = tbTask.Text;
            if (openFileDialog1.ShowDialog(this) == System.Windows.Forms.DialogResult.OK) {
                tbTask.Text = openFileDialog1.FileName;
                try {
                    _iCurrTitleElement.ChangeTask(tbTask.Text, ETaskType.eTaskUnknown);
                }
                catch (Exception err) {
                    MessageBox.Show(err.Message);
                }
            }
        }

        private void Reload_Click(object sender, EventArgs e) {
            if (_iCurrTitleElement == null)
                return;
            try {
                _iCurrTitleElement.ChangeTask(tbTask.Text, ETaskType.eTaskUnknown);
            }
            catch (Exception err) {
                MessageBox.Show(err.Message);
            }
        }

        private void btLocationSet_Click(object sender, EventArgs e) {
            if (_iCurrTitleElement == null)
                return;
            try {
                var arr = new int[2];
                arr[0] = (int)udLocationX.Value;
                arr[1] = (int)udLocationY.Value;
                _iCurrTitleElement["Location"] = arr;
            }
            catch (Exception err) {
                MessageBox.Show(err.Message);
            }
            UpdateProperties();
        }

        private void btSizeSet_Click(object sender, EventArgs e) {
            if (_iCurrTitleElement == null)
                return;
            try {
                var arr = new int[2];
                arr[0] = (int)udSizeW.Value;
                arr[1] = (int)udSizeH.Value;
                _iCurrTitleElement["Size"] = arr;
            }
            catch (Exception err) {
                MessageBox.Show(err.Message);
            }
            UpdateProperties();
        }

        string[] _propertiesNames = new string[] {   "Type"      
                                                    ,"Name"      
                                                    ,"Location"  
                                                    ,"Size"      
                                                    ,"ZOrder"    
                                                    ,"RealZOrder"
                                                    ,"Alpha"     
                                                    ,"LogEnabled"
                                                    
                                                    ,"FileName"  
                                                    ,"Duration"  
                                                    ,"AlignAndStretch"
                                                    ,"HAlignment"
                                                    ,"VAlignment"
                                                    ,"Stretch"   
                                                    ,"Loop"      
                                                    ,"AutoSize"  
                                                    ,"WithAudio" 
                                                    };

        /// <summary>
        /// ShowProperties_Click - Вынимает все стандартные(известные) свойства и 
        /// записывает их в TextBox по одному на строку в формате:
        ///     Property: 'ИмяСвойства', type: ТипСвойства, value: ТекущееЗначение
        /// 
        /// Если свойство с запрашиваемым именем не существует у данного Титровального Элемента,
        /// то выводится строка вида:
        ///     Property: 'ИмяСвойства', Exception: [Ошибка в элементе 'ИмяЭлемента'] Свойство с именем 'ИмяСвойства' не существует.
        ///     
        /// Список имен свойств берётся из массива _propertiesNames
        /// 
        /// Краткое описание стандартных свойств, их имена и типы:
        /// 0) Тип элемента 
        ///     Property Name: "Type" type: String (ReadOnly)  Показывает тип элемента
        /// 1) Имя элемента.
        ///     Property Name: "Name" type: String
        /// 2) Левая верхняя точка охватывающего прямоугольника элемента на экране телевизора.
        ///     Property Name: "Location" type: int[2] - массив из 2-х int.
        /// 3) Размер охватывающего прямоугольника элемента на экране телевизора.
        ///     Property Name: "Size" type: int[2] - массив из 2-х int.
        /// 4) Расположение региона по Z координате. Если 0 - Распределяется автоматом, любое другое число (в том числе отрицательное) - Абсолютное значение.
        ///     Property Name: "ZOrder" type: int
        /// 5) Фактическое расположение региона по Z координате. Назначается во время инициализации.
        ///     Property Name: "RealZOrder" type: int (ReadOnly)
        /// 6) Значение непрозрачности элемента.
        ///     Property Name: "Alpha" type: int в диапозоне [0..255]. 0 - полностью прозрачно, 255 - непрозрачно
        /// 7) Разрешить регистрацию хода выполнения задания в журнал регистрации.
        ///     Property Name: "LogEnabled" type: int в диапозоне [0..1]. 0 - запретить, 1 - разрешить.
        ///     
        /// Следующие свойства могут отсутсвовать у некоторых элементов, но называются именно так!
        /// 
        /// 8) Задание. Исторически сложилось, что это свойство называется "FileName", но некоторые элементы понимают строки (RollCrawl, Caption...)
        ///     Property Name: "FileName" type: String
        /// 9) Длительность текущего задания (одного цикла) в формате hh:mm:ss.ms .
        ///     Property Name: "Duration" type: String (ReadOnly)
        /// 10) Тип выравнивания и масштабирования. Может быть 'к прямоугольнику' или 'к экрану'.
        ///     Property Name: "AlignAndStretch" type: int в диапозоне [0..1] 0 - к прямоугольнику, 1 - к экрану
        /// 11) Выравнивание картинки по горизонтали относительно региона или экрана.
        ///     Property Name: "HAlignment" type: int в диапозоне [0..2] - center, left, right
        /// 12) Выравнивание картинки по вертикали относительно региона или экрана.
        ///     Property Name: "VAlignment" type:  int в диапозоне [0..2] - center, top, bottom
        /// 13) Масштабирование картинки к размерам элемента или экрана.
        ///     Property Name: "Stretch" type: int в диапозоне [0..4] - eStretchNone = 0, eStretchAll = 1, eStretchSafeAspectRatio = 2, eStretchAllByField = 3, eStretchSafeAspectRatioByField = 4
        /// 14) Зациклить задание.
        ///     Property Name: "Loop" type: int - 0, 1
        /// 15) Автоматическое изменение размера элемента по размеру картинки.   
        ///     Property Name: "AutoSize" type: int - 0, 1
        /// 16) Играть звуковую дорожку.
        ///     Property Name: "WithAudio" type: int 0, 1
        ///     
        /// Кроме того у элементов могут присутсвовать 'свои' свойства.
        /// Их названия и типы можно узнать так:
        /// 1. В FDTitleDesigner создать объект с интересующим вас элементом
        /// 2. у нужных вам свойств поменять значения на 'не дефолтные' (иначе при сохранении они не сохранятся)
        /// 3. сохранить проект
        /// 4. открыть файл сохранённого проекта (.SLTitleProj) в любом текстовом редакторе (например notepad.exe),
        ///    найти интересующий вас элемент (по имени) и помотреть список сохранённых свойств
        ///    каждое свойство выглядет примерно так:
        ///        <![CDATA[<!--The default duration of advertisement without it's own duration (shots).-->
        ///        <DefDuration value="100" />]]>
        ///    собственно "DefDuration" и есть название свойства (это для элемента "MovieList"/"РекламныйБлок").
        ///    Диапозон и тип свойства можно определить в окне свойств FDTitleDesigner
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ShowProperties_Click(object sender, EventArgs e) {
            var str = new StringBuilder();
            foreach (var name in _propertiesNames) {
                str.AppendFormat("Property: '{0}', ", name);
                try {
                    object val = _iCurrTitleElement[name];
                    str.AppendFormat("type: {0}, ", val.GetType().Name);
                    if (val is Array) {
                        str.Append("values: ");
                        var arr = val as Array;
                        for (int i = 0; i < arr.Length; i++) {
                            str.AppendFormat("{0} ", arr.GetValue(i).ToString());
                        }
                    }
                    else {
                        str.AppendFormat("value: {0}", val.ToString());
                    }
                }
                catch (Exception err) {
                    string msg = err.Message.Replace("\n\r", " ").Replace("\r\n", " ").Replace("\n", " ").Replace("\r", " ");
                    str.AppendFormat("Exception: {0}", msg);
                }
                str.AppendLine();
            }
            tbProperties.Text = str.ToString();

        }

		private void ElementsForm_FormClosed(object sender, FormClosedEventArgs e)
		{
			ClearAll();
		}

    }
}
