using System;
using System.IO;
using System.Runtime.InteropServices;
//using SLTMTitleCommon;

//using LogotypeLib;


namespace SLTMTitleRunTime
{
    [Guid("1E70B792-7D43-416c-80A3-B75B65A82F8F")]
    [ComVisible(true)]
    public enum SLTMRunTimeStatus
    {
        eNone = -1,
        eStopping,
        ePreparing,
        eStopped,
        eStarting,
        eRunning
    }

	[ComVisible(true)]
	[Guid("3A060AC1-B56E-4042-93DC-1B3CADAA8850")]
	[Flags]
	public enum EMediaPresent
	{
		MediaPresentNone = 0,
		VideoPresent = 1,
		AudioPresent = 2,
		AlphaPresent = 4,
	}

	[ComVisible(true)]
	[Guid("B526FF00-D276-499F-8D91-C65B2F96AE87")]
	public enum ETaskType
	{
		eTaskUnknown = -1,
		eTaskFileName = 0,
		eTaskText = 1,
		eTaskXML = 2,
	}

    [ComVisible(true)]
    [Guid("84635B24-8B4F-4dae-941D-7B53EA871762")]
	public enum SLTMTitleTransitionType {
		eNone = 0,
		eFadeIn,
		eFadeOut
	}

	//[ComVisible(false)]
	//[ComVisible(true)]
	[InterfaceType(ComInterfaceType.InterfaceIsIUnknown)]
	[Guid("F1C90825-5CF3-41d4-A9F4-0B15A299319D")]
    public interface ISLTitleServerTargetMgr
    {
		int get_TargetsCount(); //[Out] out int pVal);
		[return:MarshalAs(UnmanagedType.IUnknown)]
		object get_TargetByIndex([In] int TargetNumber);//, [Out] out object Unk);
	};

	
	[InterfaceType(ComInterfaceType.InterfaceIsIUnknown)]
	[Guid("919956A7-229D-48ea-842E-AC2B809EC5C8")]
    [ComVisible(true)]
    public interface ISLTMTitleRunTimeMgr
    {
		/// <summary>
		/// Init - load cofiguration file for spicified 'ForwardT' board.
		/// </summary>
		/// <param name="fileName"></param>
		/// <param name="boardIndex"></param>
		void Init([In, MarshalAs(UnmanagedType.BStr)] String fileName, int boardIndex);
		
		/// <summary>
		/// Uninit - Unload all cofiguration and Release all resources.
		/// </summary>
		void Uninit();
		
		/// <summary>
		/// NumObjects - return number of titling objects in the current configuration.
		/// </summary>
		int NumObjects {get;}
		
		/// <summary>
		/// GetName - return the name of the titling object spicified by the 'index' param
		/// </summary>
		/// <param name="index">Required titling object</param>
		/// <returns>the name of the titling object</returns>
		[return:MarshalAs(UnmanagedType.BStr)]
		String GetName([In] int index);

		/// <summary>
		/// GetRunTimeInterface - return pointer to ISLTMRunTimeInteface correponded by the 'index' param
		/// </summary>
		/// <param name="index">Required titling object</param>
		/// <returns>pointer to ISLTMRunTimeInteface</returns>
		[return:MarshalAs(UnmanagedType.IUnknown)]
		Object GetRunTimeInterface([In] int index);
	}

	[InterfaceType(ComInterfaceType.InterfaceIsIUnknown)]
	[Guid("2EAD6096-476B-4b48-815E-0434E26B8190")]
    [ComVisible(true)]
    public interface ISLTMTitleRunTimeMgr2
    {
		/// <summary>
		/// Init - load configuration file for specified 'ForwardT' board.
		/// </summary>
		/// <param name="fileName"></param>
		/// <param name="boardIndex"></param>
		void Init([In, MarshalAs(UnmanagedType.BStr)] String fileName, [In] int boardIndex, [In] ref Guid targetGuid);
		
		/// <summary>
		/// Uninit - Unload all cofiguration and Release all resources.
		/// </summary>
		void Uninit();
		
		/// <summary>
		/// NumObjects - return number of titling objects in the current configuration.
		/// </summary>
		int NumObjects {get;}
		
		/// <summary>
		/// GetName - return the name of the titling object spicified by the 'index' param
		/// </summary>
		/// <param name="index">Required titling object</param>
		/// <returns>the name of the titling object</returns>
		[return:MarshalAs(UnmanagedType.BStr)]
		String GetName([In] int index);

		/// <summary>
		/// GetRunTimeInterface - return pointer to ISLTMRunTimeInteface correponded by the 'index' param
		/// </summary>
		/// <param name="index">Required titling object</param>
		/// <returns>pointer to ISLTMRunTimeInteface</returns>
		[return:MarshalAs(UnmanagedType.IUnknown)]
		Object GetRunTimeInterface([In] int index);
	}

    /// <summary>
    ///        SLTitleServerTarget
    /// </summary>
    [InterfaceType(ComInterfaceType.InterfaceIsIUnknown)]
    [Guid("21FC2C05-6C0F-44ab-BB21-F294F0D67F12")]
    [ComVisible(true)]
    public interface ISLTMTitleRunTimeMgr3
    {
        /// <summary>
        /// Init - load configuration file with specified TitleTarget object
        /// </summary>
        /// <param name="fileName"></param>
        /// <param name="titleTarget"></param>
        void Init([In, MarshalAs(UnmanagedType.BStr)] String fileName, [In, MarshalAs(UnmanagedType.IUnknown)] object titleTarget);
    }

    /// <summary>
    ///        SLTitleServerTarget
    ///     zOrder
    /// </summary>
    [InterfaceType(ComInterfaceType.InterfaceIsIUnknown)]
    [Guid("74368272-C910-4409-8BB4-85247D08683E")]
    [ComVisible(true)]
    public interface ISLTMTitleRunTimeMgr4
    {
        /// <summary>
        /// Init - load configuration file with specified TitleTarget object and specified ZOrder.
        /// </summary>
        /// <param name="fileName"></param>
        /// <param name="titleTarget"></param>
        /// <param name="zOrderBase"></param>
        void Init2([In, MarshalAs(UnmanagedType.BStr)] String fileName, [In, MarshalAs(UnmanagedType.IUnknown)] object titleTarget, [In] Int32 zOrderBase);
        /// <summary>
        /// Init - load configuration file for specified 'ForwardT' board and specified ZOrder.
        /// </summary>
        /// <param name="fileName"></param>
        /// <param name="boardIndex"></param>
        void Init([In, MarshalAs(UnmanagedType.BStr)] String fileName, [In] int boardIndex, [In] ref Guid targetGuid, [In] Int32 zOrderBase);
    }

    [InterfaceType(ComInterfaceType.InterfaceIsIUnknown)]
    [Guid("b2c577e3-fcea-409c-a404-455358ef5cd8")]
    [ComVisible(true)]
    public interface ISLTMWarnings
    {
        /// <summary>
        /// Disable/Enable show warning boxes and set default answer for 'Ignore'/'Cancel' qwestions.
        /// </summary>
        /// <param name="suppress">disable/enable warning</param>
        /// <param name="IgnoreWarning">default answer</param>
        void SuppressWarningBoxes([In, MarshalAs(UnmanagedType.Bool)] bool suppress, [In, MarshalAs(UnmanagedType.Bool)] bool ignoreWarning);
    }

    [InterfaceType(ComInterfaceType.InterfaceIsIUnknown)]
    [Guid("BEF1FA2D-07E2-4890-802C-903A5D2776A4")]
    [ComVisible(true)]
    public interface ISLTMTitleSecurity
    {
        /// <summary>
        ///  SoftID.      Init-.
        /// </summary>
        /// <param name="nSoftID"></param>
        void SetSoftwareID([In] int nSoftID);

        /// <summary>
        /// CheckSoftwareID - check software ID for the current title engine.
        /// </summary>
        /// <param name="nSoftID"></param>
        /// <returns></returns>
        [return: MarshalAs(UnmanagedType.Bool)]
        bool CheckSoftwareID([In] int nSoftID);

        /// <summary>
        /// CheckSoftwareID - check software ID for specified 'ForwardT' board.
        /// </summary>
        /// <param name="nSoftID"></param>
        /// <param name="fileName"></param>
        /// <param name="boardIndex"></param>
        /// <returns></returns>
        [return: MarshalAs(UnmanagedType.Bool)]
        bool CheckSoftwareID([In] int nSoftID, [In] int boardIndex, [In] ref Guid targetGuid);
        
        /// <summary>
        /// CheckSoftwareID - check software ID with specified TitleTarget object
        /// </summary>
        /// <param name="nSoftID"></param>
        /// <param name="fileName"></param>
        /// <param name="titleTarget"></param>
        /// <returns></returns>
        [return: MarshalAs(UnmanagedType.Bool)]
        bool CheckSoftwareID([In] int nSoftID, [In, MarshalAs(UnmanagedType.IUnknown)] object titleTarget);
    }

    /// <summary>
    /// ISLTMTitleAudio -     
    /// </summary>
    [InterfaceType(ComInterfaceType.InterfaceIsIUnknown)]
    [ComVisible(true)]
    [Guid("1cee454e-423a-4b72-9fb4-afc6707f643c")]
    public interface ISLTMTitleAudio
    {
        /// <summary>
        ///       (-96.0 - +12.0)
        /// </summary>
        /// <value>    -96.0   +12.0 </value>
        double Volume { get; set; }

        /// <summary>
        /// Mute - / 
        /// </summary>
        /// <value>/ </value>
        bool Mute { 
            [return: MarshalAs(UnmanagedType.Bool)] get;
            [param: MarshalAs(UnmanagedType.Bool)] set;
        }
    }

    /// <summary>
	/// ISLTMTitleRunTime -      
	/// </summary>
	[InterfaceType(ComInterfaceType.InterfaceIsIUnknown)]
	[Guid("8C3650DF-DB90-46b4-9D5E-75C9C11C8FA1")]
	[ComVisible(true)]
	public interface ISLTMTitleRunTime {
		/// <summary>
		/// property Name - return name of the Titling object represented of this instance
		/// </summary>
		String Name{ get; }

		/// <summary>
		///	 property Status -   .
		/// </summary>
		SLTMRunTimeStatus Status { get; }
		
		/// <summary>
		/// method Start - ()       
		/// </summary>
		/// <param name="">
		///  
		/// (0 -  ;  -  ;  -  .. 
		/// </param>
		/// <param name="tDuration">  ( )</param>
		void Start([In] Double tStart, [In] Double tDuration);
		
		/// <summary>
		/// method Stop -  () -    () 
		///   ( ,   -    )
		/// </summary>
		/// <param name="tStart">
		///  
		/// (0 -  ;  -  ;  -  .. 
		/// </param>
		/// <param name="tDuration">  ( )</param>
		void Stop([In] Double tStart, [In] Double tDuration);
		
		/// <summary>
		/// method Abort -   -   .
		/// </summary>
		/// <param name="tDuration">  ( )</param>
		void Abort([In] Double tDuration);
		
		/// <summary>
		/// method GetFileMask -    ()
		/// </summary>
		String TaskFileMask {[return:MarshalAs(UnmanagedType.BStr)]get;}
		
		/// <summary>
		/// method LoadTask -  
		/// </summary>
		/// <param name="file"></param>
		void LoadTask([In, MarshalAs(UnmanagedType.BStr)] String file);

		/// <summary>
		/// return FileName of loadedTask
		/// </summary>
		String TaskFileName {[return:MarshalAs(UnmanagedType.BStr)]get;}
	}
		
	/// <summary>
	/// ISLTMTitleRunTime2 -      
	/// </summary>
	[InterfaceType(ComInterfaceType.InterfaceIsIUnknown)]
	[Guid("5B29BA74-341C-4466-BF22-3C683C92FF96")]
	[ComVisible(true)]
	public interface ISLTMTitleRunTime2 {
		
		#region from ISLTMTitleRunTime

		/// <summary>
		/// property Name - return name of the Titling object represented of this instance
		/// </summary>
		String Name{ get; }

		/// <summary>
		///	 property Status -   .
		/// </summary>
		SLTMRunTimeStatus Status { get; }
		
		/// <summary>
		/// method Start - ()       
		/// </summary>
		/// <param name="">
		///  
		/// (0 -  ;  -  ;  -  .. 
		/// </param>
		/// <param name="tDuration">  ( )</param>
		void Start([In] Double tStart, [In] Double tDuration);
		
		/// <summary>
		/// method Stop -  () -    () 
		///   ( ,   -    )
		/// </summary>
		/// <param name="tStart">
		///  
		/// (0 -  ;  -  ;  -  .. 
		/// </param>
		/// <param name="tDuration">  ( )</param>
		void Stop([In] Double tStart, [In] Double tDuration);
		
		/// <summary>
		/// method Abort -   -   .
		/// </summary>
		/// <param name="tDuration">  ( )</param>
		void Abort([In] Double tDuration);
		
		/// <summary>
		/// method GetFileMask -    ()
		/// </summary>
		String TaskFileMask {[return:MarshalAs(UnmanagedType.BStr)]get;}
		
		/// <summary>
		/// method LoadTask -  
		/// </summary>
		/// <param name="file"></param>
		void LoadTask([In, MarshalAs(UnmanagedType.BStr)] String file);

		/// <summary>
		/// return FileName of loadedTask
		/// </summary>
		String TaskFileName {[return:MarshalAs(UnmanagedType.BStr)]get;}

		#endregion

		bool Loopped { [return:MarshalAs(UnmanagedType.Bool)] get; [param:MarshalAs(UnmanagedType.Bool)] set; }

		/// <summary>
		///           fileName.
		///         InvalidArg.
		/// </summary>
		/// <param name="fileName">     </param>
		/// <param name="loop">  ,    
		/// (            /).</param>
		/// <returns>  (duration)     .</returns>
		double GetTaskInfo([In, MarshalAs(UnmanagedType.BStr)] string fileName, [Out, MarshalAs(UnmanagedType.Bool)] out bool loop);

		/// <summary>
		///   
		/// </summary>
		/// <param name="currTime">     (    //).</param>
		/// <param name="restTime">     ( )</param>
		/// <param name="allDuration">  (duration)     .</param>
		/// <param name="loopped">  (            /)</param>
		void GetPlayInfo([Out] out double currTime, [Out] out double restTime, [Out] out double allDuration, [Out, MarshalAs(UnmanagedType.Bool)] out bool loopped);
	}
	
	/// <summary>
	/// ISLTMTitleRunTime3 -      
	/// </summary>
	[InterfaceType(ComInterfaceType.InterfaceIsIUnknown)]
	[Guid("5CB4EFC1-84DB-489f-9971-B916AB37185E")]
	[ComVisible(true)]
	public interface ISLTMTitleRunTime3 {
		
		#region ISLTMTitleRunTime2 members
		
		#region from ISLTMTitleRunTime

		/// <summary>
		/// property Name - return name of the Titling object represented of this instance
		/// </summary>
		String Name{ get; }

		/// <summary>
		///	 property Status -   .
		/// </summary>
		SLTMRunTimeStatus Status { get; }
		
		/// <summary>
		/// method Start - ()       
		/// </summary>
		/// <param name="">
		///  
		/// (0 -  ;  -  ;  -  .. 
		/// </param>
		/// <param name="tDuration">  ( )</param>
		void Start([In] Double tStart, [In] Double tDuration);
		
		/// <summary>
		/// method Stop -  () -    () 
		///   ( ,   -    )
		/// </summary>
		/// <param name="tStart">
		///  
		/// (0 -  ;  -  ;  -  .. 
		/// </param>
		/// <param name="tDuration">  ( )</param>
		void Stop([In] Double tStart, [In] Double tDuration);
		
		/// <summary>
		/// method Abort -   -   .
		/// </summary>
		/// <param name="tDuration">  ( )</param>
		void Abort([In] Double tDuration);
		
		/// <summary>
		/// method GetFileMask -    ()
		/// </summary>
		String TaskFileMask {[return:MarshalAs(UnmanagedType.BStr)]get;}
		
		/// <summary>
		/// method LoadTask -  
		/// </summary>
		/// <param name="file"></param>
		void LoadTask([In, MarshalAs(UnmanagedType.BStr)] String file);

		/// <summary>
		/// return FileName of loadedTask
		/// </summary>
		String TaskFileName {[return:MarshalAs(UnmanagedType.BStr)]get;}

		#endregion

		bool Loopped { [return:MarshalAs(UnmanagedType.Bool)] get; [param:MarshalAs(UnmanagedType.Bool)] set; }

		/// <summary>
		///           fileName.
		///         InvalidArg.
		/// </summary>
		/// <param name="fileName">     </param>
		/// <param name="loop">  ,    
		/// (            /).</param>
		/// <returns>  (duration)     .</returns>
		double GetTaskInfo([In, MarshalAs(UnmanagedType.BStr)] string fileName, [Out, MarshalAs(UnmanagedType.Bool)] out bool loop);

		/// <summary>
		///   
		/// </summary>
		/// <param name="currTime">     (    //).</param>
		/// <param name="restTime">     ( )</param>
		/// <param name="allDuration">  (duration)     .</param>
		/// <param name="loopped">  (            /)</param>
		void GetPlayInfo([Out] out double currTime, [Out] out double restTime, [Out] out double allDuration, [Out, MarshalAs(UnmanagedType.Bool)] out bool loopped);
		
		#endregion

		/// <summary>
		///   : 
		///		   (  S_OK == 0)
		///		     (hr)      S_OK,  
		///		 errorMsg   ,
		///           fileName.
		///         InvalidArg.
		/// </summary>
		/// <param name="fileName">     </param>
		/// <param name="loop">  ,    
		/// (            /).</param>
		/// <param name="hr">  .</param>
		/// <param name="errorMsg">  </param>
		/// <returns>  (duration)     .</returns>
		double GetTaskInfoNew(
			[In, MarshalAs(UnmanagedType.BStr)] string fileName, 
			[Out, MarshalAs(UnmanagedType.Bool)] out bool loop,
			[Out, MarshalAs(UnmanagedType.Error)] out UInt32 hr,
			[Out, MarshalAs(UnmanagedType.BStr)] out String errorMsg);
	}

    /// <summary>
    /// ISLTMTitleRunTime4 -      
    /// </summary>
    [InterfaceType(ComInterfaceType.InterfaceIsIUnknown)]
    [Guid("89E05A1D-AC00-4a17-87C7-5BF0CF6EAC51")]
    [ComVisible(true)]
    public interface ISLTMTitleRunTime4
    {

        #region ISLTMTitleRunTime3 members
        #region ISLTMTitleRunTime2 members
        #region from ISLTMTitleRunTime

        /// <summary>
        /// property Name - return name of the Titling object represented of this instance
        /// </summary>
        String Name { get; }

        /// <summary>
        ///	 property Status -   .
        /// </summary>
        SLTMRunTimeStatus Status { get; }

        /// <summary>
        /// method Start - ()       
        /// </summary>
        /// <param name="">
        ///  
        /// (0 -  ;  -  ;  -  .. 
        /// </param>
        /// <param name="tDuration">  ( )</param>
        void Start([In] Double tStart, [In] Double tDuration);

        /// <summary>
        /// method Stop -  () -    () 
        ///   ( ,   -    )
        /// </summary>
        /// <param name="tStart">
        ///  
        /// (0 -  ;  -  ;  -  .. 
        /// </param>
        /// <param name="tDuration">  ( )</param>
        void Stop([In] Double tStart, [In] Double tDuration);

        /// <summary>
        /// method Abort -   -   .
        /// </summary>
        /// <param name="tDuration">  ( )</param>
        void Abort([In] Double tDuration);

        /// <summary>
        /// method GetFileMask -    ()
        /// </summary>
        String TaskFileMask { [return: MarshalAs(UnmanagedType.BStr)]get; }

        /// <summary>
        /// method LoadTask -  
        /// </summary>
        /// <param name="file"></param>
        void LoadTask([In, MarshalAs(UnmanagedType.BStr)] String file);

        /// <summary>
        /// return FileName of loadedTask
        /// </summary>
        String TaskFileName { [return: MarshalAs(UnmanagedType.BStr)]get; }

        #endregion

        bool Loopped { [return: MarshalAs(UnmanagedType.Bool)] get; [param: MarshalAs(UnmanagedType.Bool)] set; }

        /// <summary>
        ///           fileName.
        ///         InvalidArg.
        /// </summary>
        /// <param name="fileName">     </param>
        /// <param name="loop">  ,    
        /// (            /).</param>
        /// <returns>  (duration)     .</returns>
        double GetTaskInfo([In, MarshalAs(UnmanagedType.BStr)] string fileName, [Out, MarshalAs(UnmanagedType.Bool)] out bool loop);

        /// <summary>
        ///   
        /// </summary>
        /// <param name="currTime">     (    //).</param>
        /// <param name="restTime">     ( )</param>
        /// <param name="allDuration">  (duration)     .</param>
        /// <param name="loopped">  (            /)</param>
        void GetPlayInfo([Out] out double currTime, [Out] out double restTime, [Out] out double allDuration, [Out, MarshalAs(UnmanagedType.Bool)] out bool loopped);

        #endregion

        /// <summary>
        ///   : 
        ///		   (  S_OK == 0)
        ///		     (hr)      S_OK,  
        ///		 errorMsg   ,
        ///           fileName.
        ///         InvalidArg.
        /// </summary>
        /// <param name="fileName">     </param>
        /// <param name="loop">  ,    
        /// (            /).</param>
        /// <param name="hr">  .</param>
        /// <param name="errorMsg">  </param>
        /// <returns>  (duration)     .</returns>
        double GetTaskInfoNew(
            [In, MarshalAs(UnmanagedType.BStr)] string fileName,
            [Out, MarshalAs(UnmanagedType.Bool)] out bool loop,
            [Out, MarshalAs(UnmanagedType.Error)] out UInt32 hr,
            [Out, MarshalAs(UnmanagedType.BStr)] out String errorMsg);
        #endregion

        /// <summary>
        ///    :-)  : 
        ///          // ( -?)
        ///		   (  S_OK == 0)
        ///		     (hr)      S_OK,  
        ///		 errorMsg   ,
        ///           fileName.
        ///         InvalidArg.
        /// </summary>
        /// <param name="fileName">     </param>
        /// <param name="loop">  ,    
        /// (            /).</param>
        /// <param name="hr">  .</param>
        /// <param name="errorMsg">  </param>
        /// <returns>  (duration)     .</returns>
        double GetTaskInfoEx(
            [In, MarshalAs(UnmanagedType.BStr)] string fileName,
            [Out, MarshalAs(UnmanagedType.Bool)] out bool loop,
            [Out, MarshalAs(UnmanagedType.Error)] out UInt32 hr,
            [Out, MarshalAs(UnmanagedType.BStr)] out String errorMsg,
            [Out] out EMediaPresent mediaPresent);

    }


    [InterfaceType(ComInterfaceType.InterfaceIsIUnknown)]
    [Guid("DFF64905-FEC6-49fe-B649-425AC442821A")]
    [ComVisible(true)]
    public interface ISLTMTitleEnumElements
    {
        /// <summary>
        /// NumObjects - return number of titling elements in the current TitleObject.
        /// </summary>
        int NumObjects { get; }

        /// <summary>
        /// GetName - return the name of the titling elemen spicified by the 'index' param
        /// </summary>
        /// <param name="index">Required titling element</param>
        /// <returns>the name of the titling element</returns>
        [return: MarshalAs(UnmanagedType.BStr)]
        String GetName([In] int index);

        /// <summary>
        /// GetRunTimeInterface - return pointer to ISLTMRunTimeInteface correponded by the 'index' param
        /// </summary>
        /// <param name="index">Required titling object</param>
        /// <returns>pointer to IElement</returns>
        [return: MarshalAs(UnmanagedType.IUnknown)]
        Object GetElement([In] int index);
    }

    /// <summary>
	/// ISLTMTitleElement
    /// </summary>
	[InterfaceType(ComInterfaceType.InterfaceIsIUnknown)]
    [Guid("26A0F2F6-7D4C-48f4-AC72-E98273BD3F27")]
    [ComVisible(true)]
    public interface ISLTMTitleElement
    {
        void Abort(double tFadeDuration);
        int AddTask([MarshalAs(UnmanagedType.BStr)]string task, ETaskType taskType);
        int ChangeTask([MarshalAs(UnmanagedType.BStr)]string task, ETaskType taskType);
        double GlobalTime { get; }
        [return: MarshalAs(UnmanagedType.BStr)] string GetTask();
        void LoadTask([MarshalAs(UnmanagedType.BStr)]string task, ETaskType taskType);
        string Name { [return: MarshalAs(UnmanagedType.BStr)]get; }
        object this[string name] { get; set; }
        void Start(double tStart, double tFadeDuration);
        SLTMRunTimeStatus Status { get; }
        double Stop(double tStop, double tFadeDuration);
        [return: MarshalAs(UnmanagedType.Bool)] bool waitStopped();
        [return: MarshalAs(UnmanagedType.BStr)] string GetTaskMask();
    }

        /// <summary>
    ///      .
    /// </summary>
    [Guid("E6685FBA-D076-4755-8295-211D615A0FA6")]
    [ComVisible(true)]
    [StructLayout(LayoutKind.Sequential)]
    public struct PlayInfo
    {
        /// <summary>
        ///  ( ),     .
        /// </summary>
        public double restTimeAll;
        /// <summary>
        ///  ( ),     .
        /// </summary>
        public double restTime;
        /// <summary>
        ///   ( )   .
        /// </summary>
        public double duration;
        /// <summary>
        ///     .
        /// </summary>
        public double globalTime;
        /// <summary>
        ///   .
        /// </summary>
        public bool loopped;
    }

        /// <summary>
    ///   .
    /// </summary>
    [Guid("204BE7DD-4602-4a65-B332-4C10122F4BF5")]
    [ComVisible(true)]
    [StructLayout(LayoutKind.Sequential)]
    public struct TaskInfo
    {
        /// <summary>
        ///   ( )   .
        /// </summary>
        public double duration;
        /// <summary>
        ///  ( ),     ,      .
        /// :   RollCrawl,    ''        .
        /// </summary>
        public double firstStop;
        /// <summary>
        ///  .
        /// </summary>
        public bool loopped;
    }

    /// <summary>
	/// ISLTMTitleElement2
    /// </summary>
	[InterfaceType(ComInterfaceType.InterfaceIsIUnknown)]
    [Guid("A0930629-DC64-49d9-9AFD-F9ACA07371DB")]
    [ComVisible(true)]
    public interface ISLTMTitleElement2 : ISLTMTitleElement {
        /// <summary>
        ///       . 
        /// </summary>
        /// <param name="info">     .</param>
        void GetPlayInfo(ref PlayInfo info);
        /// <summary>
        ///    .
        /// </summary>
        /// <param name="task"></param>
        /// <param name="taskType"> </param>
        /// <param name="info">     .</param>
        void GetTaskInfo(string task, ETaskType taskType, ref TaskInfo info);
        /// <summary>
        ///  .
        /// </summary>
        /// <remarks>  ,    . 
        ///       ,   ,    <see cref="SLTMRunTimeStatus.ePreparing"/></remarks>
        /// <param name="task">.</param>
        /// <param name="taskType"> .</param>
        /// <returns>   (<c>taskId</c>).         <see cref="ScriptBase.OnElementStart"/>,<see cref="ScriptBase.OnElementFinishing"/>  <see cref="ScriptBase.OnElementStop"/></returns>
		new int LoadTask(string task, ETaskType taskType);
    }
    /// <summary>
    /// ISLTMTitleBackGroundNotify
    /// </summary>
    [InterfaceType(ComInterfaceType.InterfaceIsIUnknown)]
    [Guid("6B4E330C-C236-4de9-BDA4-C0D3CEA07186")]
    [ComVisible(true)]
    public interface ISLTMTitleBackGroundNotify {
        /// <summary>
        ///         BackGroundVideo
        /// </summary>
        /// <returns></returns>
        [return: MarshalAs(UnmanagedType.Bool)] bool NeedNotify();
        /// <summary>
        ///      BackGroundVideo
        /// </summary>
        /// <param name="info"> xml,   ( )</param>
        void OnBackGroundNotify([MarshalAs(UnmanagedType.BStr)] string info);
    }

	/// <summary>
	/// ISLTMTitleCommonAlpha -      
	/// </summary>
	[InterfaceType(ComInterfaceType.InterfaceIsIUnknown)]
	[Guid("ED949410-EA05-473d-95D5-E2E8B2DD4595")]
	[ComVisible(true)]
	public interface ISLTMTitleCommonAlpha {

		/// <summary>
		/// get & set common alpha for titleObject (0..255)
		/// </summary>
		Int32 Alpha { get; set; }

		/// <summary>
		/// Interpolate -     atTime  durationTime,     toAlpha 
		/// </summary>
		/// <param name="atTime">    . 
		///  0 -  , . -  atTime ., . -  
		/// </param>
		/// <param name="durationTime">   .</param>
		/// <param name="toAlpha">   (0..255)</param>
		void Interpolate([In] double atTime, [In] double durationTime, [In] Int32 toAlpha);
	}

	/// <summary>
	/// ISLTMTitleObjectProperties -      
	/// (   )
	/// 24.11.2014      '' - (x86-x64, x64-x86)
	///    32- ( ), 
	///    IntPtr  Int32,   [Obsolete] 
	///    <see cref="ISLTMTitleObjectProperties2"/>   Int64
	/// </summary>
	[InterfaceType(ComInterfaceType.InterfaceIsIUnknown)]
	[Guid("230436AE-5CBE-42f5-B5C1-74C8EC3604EA")]
	[ComVisible(true)]
	[Obsolete]
	public interface ISLTMTitleObjectProperties {
		/// <summary>
		/// Show the modal dialog box with properties of all Title Elemnts
		/// </summary>
		void ShowDialog(Int32 hwnd);
		/// <summary>
		/// Show the modelles dialog box with properties of all Title Elemnts
		/// </summary>
		void Show(Int32 hwnd);
	}

	/// <summary>
	/// ISLTMTitleObjectProperties2 -      
	/// (   )
	/// 24.11.2014       <see cref="ISLTMTitleObjectProperties"/>,  '' - (x86-x64, x64-x86)
	///    (ISLTMTitleObjectProperties)   Int64
	/// </summary>
	[InterfaceType(ComInterfaceType.InterfaceIsIUnknown)]
	[Guid("933AEF1D-6F6E-42B6-9D39-0A09A4564E0B")]
	[ComVisible(true)]
	public interface ISLTMTitleObjectProperties2
	{
		/// <summary>
		/// Show the modal dialog box with properties of all Title Elemnts
		/// </summary>
		void ShowDialog(Int64 hwnd);
		/// <summary>
		/// Show the modelles dialog box with properties of all Title Elemnts
		/// </summary>
		void Show(Int64 hwnd);
	}

	/// <summary>
    /// ISLTMTitleLog -  Logging object
	/// </summary>
	[InterfaceType(ComInterfaceType.InterfaceIsIUnknown)]
	[Guid("534687CB-97E4-44af-87E8-A80B6A068FB6")]
	[ComVisible(true)]
	public interface ISLTMTitleLog {
		void SetLogObject([In, MarshalAs(UnmanagedType.IUnknown)] object iSLLog);
	}

    /// <summary>
    /// ISLTMTitleScriptDebug -  
    /// </summary>
    [InterfaceType(ComInterfaceType.InterfaceIsIUnknown)]
    [Guid("32250B2E-2395-44df-96A0-F1EF276DB923")]
    [ComVisible(true)]
    public interface ISLTMTitleScriptDebug {
		/// <summary>
		///  
		/// </summary>
        bool EnableScriptTrace { [return: MarshalAs(UnmanagedType.Bool)] get; [param: MarshalAs(UnmanagedType.Bool)] set; }
    }

	/// <summary>
	/// ISLTMTitlePrimaryInit
	/// </summary>
	[InterfaceType(ComInterfaceType.InterfaceIsIUnknown)]
	[Guid("A5177C2A-27A8-4ACF-B841-49D13996947B")]
	[ComVisible(true)]
	public interface ISLTMTitlePrimaryInit
	{
		/// <summary>
		///   /  
		///       '' 
		/// </summary>
		/// <param name="lcid"> . : 0x409 - "en-us", 0x419 - "ru-ru"</param>
		/// <param name="guardEventName">  </param>
		void PrimaryInit(int lcid, [MarshalAs(UnmanagedType.BStr)] string guardEventName);
	}
}
