#include "StdAfx.h"
#include "CrawlGenerator.h"

HRESULT CCrawlGenerator::Init(RECT rect, long nZOrder)
{
	HRESULT hres = CImageGenerator::Init(rect, nZOrder);
	if( FAILED(hres) )  return hres;
	// allocate buffer for moving image
	m_pBaseImage = new RGBQUAD[(rect.bottom - rect.top) * nImgWidth];
	if( m_pBaseImage==NULL )  return E_OUTOFMEMORY;
	// fill moving image by color gradient
	RGBQUAD *ptr = m_pBaseImage;
	for( int y=0; y<rect.bottom - rect.top; y++ ) {
		for( int x=0; x<nImgWidth; x++ ) {
			ptr[x].rgbBlue = 0;			// no blue
			ptr[x].rgbGreen = x;		// interpolate green horizontally
			ptr[x].rgbRed = 255-y;		// interpolate red vertically
			ptr[x].rgbReserved = BYTE(192);	// semitransparent image
		}
		ptr += nImgWidth;
	}
	// all right
	m_nOffset = 0;
	return S_OK;
}

HRESULT CCrawlGenerator::Release(bool no_check)
{
	HRESULT hres = CImageGenerator::Release(no_check);
	// delete local buffer
	if( m_pBaseImage != NULL ) {
		delete[] m_pBaseImage;
		m_pBaseImage = NULL;
	}
	return hres;
}

HRESULT CCrawlGenerator::DrawNextImage(RGBQUAD *image, long nFrameIndex)
{
	// calc offset of first and second fields of new frame (shot) image
	long nOff1 = m_nOffset % nImgWidth;
	long nOff2 = (m_nOffset + m_nStep) % nImgWidth;
	m_nOffset = (m_nOffset + m_nStep*2) % nImgWidth;
	// swap fields offset if lower field first
	if( !m_bUpperField ) {
		long tmp = nOff2;
		nOff2 = nOff1;
		nOff1 = tmp;
	}
	// copy upper field of cicled local buffer to next frame buffer
	CopyOneField(image, nOff1);
	// copy lower field of cicled local buffer to next frame buffer
	CopyOneField(image + m_nDX, nOff2);
	// image ready
	return S_OK;
}

void CCrawlGenerator::CopyOneField(RGBQUAD *pImage, long nOff)
{
	for( long x=0; x<m_nDX; ) {
		int ln = m_nDX - x; // calc rest of target image width
		if( ln > nImgWidth - nOff ) {
			// rest of source image width is lower then rest of target image width
			ln = nImgWidth - nOff;
		}
		// copy rectangle area from source image to target image
		RGBQUAD *from = &m_pBaseImage[nOff];
		RGBQUAD *to = pImage;
		for( int y=0; y<m_nDY; y += 2 ) { // copy half lines (one field only)
			// copy one line
			CopyMemory(to,from,ln*4);
			// shift source & target pointer to next line in the field (one lione more)
			from += nImgWidth*2;
			to += m_nDX*2;
		}
		// 'ln' pixels (column) copied - shift pointers to 'ln' pixels right
		pImage += ln;
		x += ln;
		// start next source image coping from the begining
		nOff = 0;
	}
}
